# Stubs for errno

from typing import Mapping
import sys

errorcode = ...  # type: Mapping[int, str]

EPERM = ...  # type: int
ENOENT = ...  # type: int
ESRCH = ...  # type: int
EINTR = ...  # type: int
EIO = ...  # type: int
ENXIO = ...  # type: int
E2BIG = ...  # type: int
ENOEXEC = ...  # type: int
EBADF = ...  # type: int
ECHILD = ...  # type: int
EAGAIN = ...  # type: int
ENOMEM = ...  # type: int
EACCES = ...  # type: int
EFAULT = ...  # type: int
ENOTBLK = ...  # type: int
EBUSY = ...  # type: int
EEXIST = ...  # type: int
EXDEV = ...  # type: int
ENODEV = ...  # type: int
ENOTDIR = ...  # type: int
EISDIR = ...  # type: int
EINVAL = ...  # type: int
ENFILE = ...  # type: int
EMFILE = ...  # type: int
ENOTTY = ...  # type: int
ETXTBSY = ...  # type: int
EFBIG = ...  # type: int
ENOSPC = ...  # type: int
ESPIPE = ...  # type: int
EROFS = ...  # type: int
EMLINK = ...  # type: int
EPIPE = ...  # type: int
EDOM = ...  # type: int
ERANGE = ...  # type: int
EDEADLCK = ...  # type: int
ENAMETOOLONG = ...  # type: int
ENOLCK = ...  # type: int
ENOSYS = ...  # type: int
ENOTEMPTY = ...  # type: int
ELOOP = ...  # type: int
EWOULDBLOCK = ...  # type: int
ENOMSG = ...  # type: int
EIDRM = ...  # type: int
ECHRNG = ...  # type: int
EL2NSYNC = ...  # type: int
EL3HLT = ...  # type: int
EL3RST = ...  # type: int
ELNRNG = ...  # type: int
EUNATCH = ...  # type: int
ENOCSI = ...  # type: int
EL2HLT = ...  # type: int
EBADE = ...  # type: int
EBADR = ...  # type: int
EXFULL = ...  # type: int
ENOANO = ...  # type: int
EBADRQC = ...  # type: int
EBADSLT = ...  # type: int
EDEADLOCK = ...  # type: int
EBFONT = ...  # type: int
ENOSTR = ...  # type: int
ENODATA = ...  # type: int
ETIME = ...  # type: int
ENOSR = ...  # type: int
ENONET = ...  # type: int
ENOPKG = ...  # type: int
EREMOTE = ...  # type: int
ENOLINK = ...  # type: int
EADV = ...  # type: int
ESRMNT = ...  # type: int
ECOMM = ...  # type: int
EPROTO = ...  # type: int
EMULTIHOP = ...  # type: int
EDOTDOT = ...  # type: int
EBADMSG = ...  # type: int
EOVERFLOW = ...  # type: int
ENOTUNIQ = ...  # type: int
EBADFD = ...  # type: int
EREMCHG = ...  # type: int
ELIBACC = ...  # type: int
ELIBBAD = ...  # type: int
ELIBSCN = ...  # type: int
ELIBMAX = ...  # type: int
ELIBEXEC = ...  # type: int
EILSEQ = ...  # type: int
ERESTART = ...  # type: int
ESTRPIPE = ...  # type: int
EUSERS = ...  # type: int
ENOTSOCK = ...  # type: int
EDESTADDRREQ = ...  # type: int
EMSGSIZE = ...  # type: int
EPROTOTYPE = ...  # type: int
ENOPROTOOPT = ...  # type: int
EPROTONOSUPPORT = ...  # type: int
ESOCKTNOSUPPORT = ...  # type: int
ENOTSUP = ...  # type: int
EOPNOTSUPP = ...  # type: int
EPFNOSUPPORT = ...  # type: int
EAFNOSUPPORT = ...  # type: int
EADDRINUSE = ...  # type: int
EADDRNOTAVAIL = ...  # type: int
ENETDOWN = ...  # type: int
ENETUNREACH = ...  # type: int
ENETRESET = ...  # type: int
ECONNABORTED = ...  # type: int
ECONNRESET = ...  # type: int
ENOBUFS = ...  # type: int
EISCONN = ...  # type: int
ENOTCONN = ...  # type: int
ESHUTDOWN = ...  # type: int
ETOOMANYREFS = ...  # type: int
ETIMEDOUT = ...  # type: int
ECONNREFUSED = ...  # type: int
EHOSTDOWN = ...  # type: int
EHOSTUNREACH = ...  # type: int
EALREADY = ...  # type: int
EINPROGRESS = ...  # type: int
ESTALE = ...  # type: int
EUCLEAN = ...  # type: int
ENOTNAM = ...  # type: int
ENAVAIL = ...  # type: int
EISNAM = ...  # type: int
EREMOTEIO = ...  # type: int
EDQUOT = ...  # type: int



# Stubs for logging.config (Python 3.4)

from typing import Any, Callable, Dict, Optional, IO, Union
import sys
if sys.version_info >= (3,):
    from configparser import RawConfigParser
else:
    from ConfigParser import RawConfigParser
if sys.version_info >= (3, 6):
    from os import PathLike

if sys.version_info >= (3, 7):
    _Path = Union[str, bytes, PathLike[str]]
elif sys.version_info >= (3, 6):
    _Path = Union[str, PathLike[str]]
else:
    _Path = str


def dictConfig(config: Dict[str, Any]) -> None: ...
if sys.version_info >= (3, 4):
    def fileConfig(fname: Union[_Path, IO[str], RawConfigParser],
                   defaults: Optional[Dict[str, str]] = ...,
                   disable_existing_loggers: bool = ...) -> None: ...
    def listen(port: int = ...,
               verify: Optional[Callable[[bytes], Optional[bytes]]] = ...) -> None: ...
else:
    def fileConfig(fname: Union[str, IO[str]],
                   defaults: Optional[Dict[str, str]] = ...,
                   disable_existing_loggers: bool = ...) -> None: ...
    def listen(port: int = ...) -> None: ...
def stopListening() -> None: ...



"""This file exists as a temporary measure, and will be removed when new
semantic analyzer is the default one.
"""
MYPY = False
if MYPY:
    from typing_extensions import Final

# Files to not run with new semantic analyzer.
new_semanal_blacklist = [
    'check-async-await.test',
    'check-expressions.test',
    'check-flags.test',
    'check-functions.test',
    'check-incremental.test',
    'check-literal.test',
    'check-overloading.test',
    'check-python2.test',
    'check-statements.test',
    'check-unions.test',
    'check-unreachable-code.test',
]  # type: Final



# Stubs for pickletools (Python 2 and 3)
import sys
from typing import Any, Callable, IO, Iterator, List, MutableMapping, Optional, Text, Tuple, Type, Union

_Reader = Callable[[IO[bytes]], Any]

if sys.version_info >= (3, 0):
    bytes_types: Tuple[Type[Any], ...]

UP_TO_NEWLINE: int
TAKEN_FROM_ARGUMENT1: int
TAKEN_FROM_ARGUMENT4: int
if sys.version_info >= (3, 3):
    TAKEN_FROM_ARGUMENT4U: int
if sys.version_info >= (3, 4):
    TAKEN_FROM_ARGUMENT8U: int

class ArgumentDescriptor(object):
    name: str
    n: int
    reader: _Reader
    doc: str
    def __init__(self, name: str, n: int, reader: _Reader, doc: str) -> None: ...

def read_uint1(f: IO[bytes]) -> int: ...
uint1: ArgumentDescriptor

def read_uint2(f: IO[bytes]) -> int: ...
uint2: ArgumentDescriptor

def read_int4(f: IO[bytes]) -> int: ...
int4: ArgumentDescriptor

if sys.version_info >= (3, 3):
    def read_uint4(f: IO[bytes]) -> int: ...
    uint4: ArgumentDescriptor

if sys.version_info >= (3, 5):
    def read_uint8(f: IO[bytes]) -> int: ...
    uint8: ArgumentDescriptor

def read_stringnl(f: IO[bytes], decode: bool = ..., stripquotes: bool = ...) -> Union[bytes, Text]: ...
stringnl: ArgumentDescriptor

def read_stringnl_noescape(f: IO[bytes]) -> str: ...
stringnl_noescape: ArgumentDescriptor

def read_stringnl_noescape_pair(f: IO[bytes]) -> Text: ...
stringnl_noescape_pair: ArgumentDescriptor

def read_string1(f: IO[bytes]) -> str: ...
string1: ArgumentDescriptor

def read_string4(f: IO[bytes]) -> str: ...
string4: ArgumentDescriptor

if sys.version_info >= (3, 3):
    def read_bytes1(f: IO[bytes]) -> bytes: ...
    bytes1: ArgumentDescriptor

    def read_bytes4(f: IO[bytes]) -> bytes: ...
    bytes4: ArgumentDescriptor

if sys.version_info >= (3, 4):
    def read_bytes8(f: IO[bytes]) -> bytes: ...
    bytes8: ArgumentDescriptor

def read_unicodestringnl(f: IO[bytes]) -> Text: ...
unicodestringnl: ArgumentDescriptor

if sys.version_info >= (3, 4):
    def read_unicodestring1(f: IO[bytes]) -> Text: ...
    unicodestring1: ArgumentDescriptor

def read_unicodestring4(f: IO[bytes]) -> Text: ...
unicodestring4: ArgumentDescriptor

if sys.version_info >= (3, 4):
    def read_unicodestring8(f: IO[bytes]) -> Text: ...
    unicodestring8: ArgumentDescriptor

def read_decimalnl_short(f: IO[bytes]) -> int: ...
def read_decimalnl_long(f: IO[bytes]) -> int: ...
decimalnl_short: ArgumentDescriptor
decimalnl_long: ArgumentDescriptor

def read_floatnl(f: IO[bytes]) -> float: ...
floatnl: ArgumentDescriptor

def read_float8(f: IO[bytes]) -> float: ...
float8: ArgumentDescriptor

def read_long1(f: IO[bytes]) -> int: ...
long1: ArgumentDescriptor

def read_long4(f: IO[bytes]) -> int: ...
long4: ArgumentDescriptor

class StackObject(object):
    name: str
    obtype: Union[Type[Any], Tuple[Type[Any], ...]]
    doc: str
    def __init__(self, name: str, obtype: Union[Type[Any], Tuple[Type[Any], ...]], doc: str) -> None: ...

pyint: StackObject
pylong: StackObject
pyinteger_or_bool: StackObject
pybool: StackObject
pyfloat: StackObject
if sys.version_info >= (3, 4):
    pybytes_or_str: StackObject
pystring: StackObject
if sys.version_info >= (3, 0):
    pybytes: StackObject
pyunicode: StackObject
pynone: StackObject
pytuple: StackObject
pylist: StackObject
pydict: StackObject
if sys.version_info >= (3, 4):
    pyset: StackObject
    pyfrozenset: StackObject
anyobject: StackObject
markobject: StackObject
stackslice: StackObject

class OpcodeInfo(object):
    name: str
    code: str
    arg: Optional[ArgumentDescriptor]
    stack_before: List[StackObject]
    stack_after: List[StackObject]
    proto: int
    doc: str
    def __init__(self, name: str, code: str, arg: Optional[ArgumentDescriptor],
                 stack_before: List[StackObject], stack_after: List[StackObject], proto: int, doc: str) -> None: ...

opcodes: List[OpcodeInfo]

def genops(pickle: Union[bytes, IO[bytes]]) -> Iterator[Tuple[OpcodeInfo, Optional[Any], Optional[int]]]: ...
def optimize(p: Union[bytes, IO[bytes]]) -> bytes: ...
if sys.version_info >= (3, 2):
    def dis(pickle: Union[bytes, IO[bytes]], out: Optional[IO[str]] = ..., memo: Optional[MutableMapping[int, Any]] = ..., indentlevel: int = ..., annotate: int = ...) -> None: ...
else:
    def dis(pickle: Union[bytes, IO[bytes]], out: Optional[IO[str]] = ..., memo: Optional[MutableMapping[int, Any]] = ..., indentlevel: int = ...) -> None: ...
